@@ -2,21 +2,18 @@ package echo
 
 import (
 	"bytes"
+	"encoding/xml"
 	"errors"
 	"io"
 	"mime/multipart"
 	"net/http"
 	"net/http/httptest"
+	"net/url"
+	"strings"
 	"testing"
 	"text/template"
 	"time"
 
-	"strings"
-
-	"net/url"
-
-	"encoding/xml"
-
 	"github.com/stretchr/testify/assert"
 )
 
@@ -217,7 +214,7 @@ func TestContext(t *testing.T) {
 	c.SetParamNames("foo")
 	c.SetParamValues("bar")
 	c.Set("foe", "ban")
-	c.query = url.Values(map[string][]string{"fon": []string{"baz"}})
+	c.query = url.Values(map[string][]string{"fon": {"baz"}})
 	c.Reset(req, httptest.NewRecorder())
 	assert.Equal(t, 0, len(c.ParamValues()))
 	assert.Equal(t, 0, len(c.ParamNames()))
@@ -303,18 +300,6 @@ func TestContextPathParam(t *testing.T) {
 	assert.Equal(t, "501", c.Param("fid"))
 }
 
-func TestContextPathParamNamesAlais(t *testing.T) {
-	e := New()
-	req := httptest.NewRequest(GET, "/", nil)
-	c := e.NewContext(req, nil)
-
-	c.SetParamNames("id,name")
-	c.SetParamValues("joe")
-
-	assert.Equal(t, "joe", c.Param("id"))
-	assert.Equal(t, "joe", c.Param("name"))
-}
-
 func TestContextFormValue(t *testing.T) {
 	f := make(url.Values)
 	f.Set("name", "Jon Snow")
