@@ -35,9 +35,9 @@ func TestGzip(t *testing.T) {
 	assert.Equal(t, gzipScheme, rec.Header().Get(echo.HeaderContentEncoding))
 	assert.Contains(t, rec.Header().Get(echo.HeaderContentType), echo.MIMETextPlain)
 	r, err := gzip.NewReader(rec.Body)
-	defer r.Close()
 	if assert.NoError(t, err) {
 		buf := new(bytes.Buffer)
+		defer r.Close()
 		buf.ReadFrom(r)
 		assert.Equal(t, "test", buf.String())
 	}
@@ -89,12 +89,13 @@ func TestGzipWithStatic(t *testing.T) {
 		assert.Equal(t, cl, rec.Body.Len())
 	}
 	r, err := gzip.NewReader(rec.Body)
-	assert.NoError(t, err)
-	defer r.Close()
-	want, err := ioutil.ReadFile("../_fixture/images/walle.png")
 	if assert.NoError(t, err) {
-		var buf bytes.Buffer
-		buf.ReadFrom(r)
-		assert.Equal(t, want, buf.Bytes())
+		defer r.Close()
+		want, err := ioutil.ReadFile("../_fixture/images/walle.png")
+		if assert.NoError(t, err) {
+			buf := new(bytes.Buffer)
+			buf.ReadFrom(r)
+			assert.Equal(t, want, buf.Bytes())
+		}
 	}
 }
