@@ -16,34 +16,34 @@ type (
 
 		// AllowOrigin defines a list of origins that may access the resource.
 		// Optional. Default value []string{"*"}.
-		AllowOrigins []string `json:"allow_origins"`
+		AllowOrigins []string `yaml:"allow_origins"`
 
 		// AllowMethods defines a list methods allowed when accessing the resource.
 		// This is used in response to a preflight request.
 		// Optional. Default value DefaultCORSConfig.AllowMethods.
-		AllowMethods []string `json:"allow_methods"`
+		AllowMethods []string `yaml:"allow_methods"`
 
 		// AllowHeaders defines a list of request headers that can be used when
 		// making the actual request. This in response to a preflight request.
 		// Optional. Default value []string{}.
-		AllowHeaders []string `json:"allow_headers"`
+		AllowHeaders []string `yaml:"allow_headers"`
 
 		// AllowCredentials indicates whether or not the response to the request
 		// can be exposed when the credentials flag is true. When used as part of
 		// a response to a preflight request, this indicates whether or not the
 		// actual request can be made using credentials.
 		// Optional. Default value false.
-		AllowCredentials bool `json:"allow_credentials"`
+		AllowCredentials bool `yaml:"allow_credentials"`
 
 		// ExposeHeaders defines a whitelist headers that clients are allowed to
 		// access.
 		// Optional. Default value []string{}.
-		ExposeHeaders []string `json:"expose_headers"`
+		ExposeHeaders []string `yaml:"expose_headers"`
 
 		// MaxAge indicates how long (in seconds) the results of a preflight request
 		// can be cached.
 		// Optional. Default value 0.
-		MaxAge int `json:"max_age"`
+		MaxAge int `yaml:"max_age"`
 	}
 )
 
