@@ -9,72 +9,81 @@ import (
 	"github.com/stretchr/testify/assert"
 )
 
+type middlewareGenerator func() echo.MiddlewareFunc
+
 func TestRedirectHTTPSRedirect(t *testing.T) {
-	e := echo.New()
-	next := func(c echo.Context) (err error) {
-		return c.NoContent(http.StatusOK)
-	}
-	req := httptest.NewRequest(echo.GET, "/", nil)
-	req.Host = "labstack.com"
-	res := httptest.NewRecorder()
-	c := e.NewContext(req, res)
-	HTTPSRedirect()(next)(c)
+	res := redirectTest(HTTPSRedirect, "labstack.com", nil)
+
 	assert.Equal(t, http.StatusMovedPermanently, res.Code)
 	assert.Equal(t, "https://labstack.com/", res.Header().Get(echo.HeaderLocation))
 }
 
+func TestHTTPSRedirectBehindTLSTerminationProxy(t *testing.T) {
+	header := http.Header{}
+	header.Set(echo.HeaderXForwardedProto, "https")
+	res := redirectTest(HTTPSRedirect, "labstack.com", header)
+
+	assert.Equal(t, http.StatusOK, res.Code)
+}
+
 func TestRedirectHTTPSWWWRedirect(t *testing.T) {
-	e := echo.New()
-	next := func(c echo.Context) (err error) {
-		return c.NoContent(http.StatusOK)
-	}
-	req := httptest.NewRequest(echo.GET, "/", nil)
-	req.Host = "labstack.com"
-	res := httptest.NewRecorder()
-	c := e.NewContext(req, res)
-	HTTPSWWWRedirect()(next)(c)
+	res := redirectTest(HTTPSWWWRedirect, "labstack.com", nil)
+
 	assert.Equal(t, http.StatusMovedPermanently, res.Code)
 	assert.Equal(t, "https://www.labstack.com/", res.Header().Get(echo.HeaderLocation))
 }
 
+func TestRedirectHTTPSWWWRedirectBehindTLSTerminationProxy(t *testing.T) {
+	header := http.Header{}
+	header.Set(echo.HeaderXForwardedProto, "https")
+	res := redirectTest(HTTPSWWWRedirect, "labstack.com", header)
+
+	assert.Equal(t, http.StatusOK, res.Code)
+}
+
 func TestRedirectHTTPSNonWWWRedirect(t *testing.T) {
-	e := echo.New()
-	next := func(c echo.Context) (err error) {
-		return c.NoContent(http.StatusOK)
-	}
-	req := httptest.NewRequest(echo.GET, "/", nil)
-	req.Host = "www.labstack.com"
-	res := httptest.NewRecorder()
-	c := e.NewContext(req, res)
-	HTTPSNonWWWRedirect()(next)(c)
+	res := redirectTest(HTTPSNonWWWRedirect, "www.labstack.com", nil)
+
 	assert.Equal(t, http.StatusMovedPermanently, res.Code)
 	assert.Equal(t, "https://labstack.com/", res.Header().Get(echo.HeaderLocation))
 }
 
+func TestRedirectHTTPSNonWWWRedirectBehindTLSTerminationProxy(t *testing.T) {
+	header := http.Header{}
+	header.Set(echo.HeaderXForwardedProto, "https")
+	res := redirectTest(HTTPSNonWWWRedirect, "www.labstack.com", header)
+
+	assert.Equal(t, http.StatusOK, res.Code)
+}
+
 func TestRedirectWWWRedirect(t *testing.T) {
-	e := echo.New()
-	next := func(c echo.Context) (err error) {
-		return c.NoContent(http.StatusOK)
-	}
-	req := httptest.NewRequest(echo.GET, "/", nil)
-	req.Host = "labstack.com"
-	res := httptest.NewRecorder()
-	c := e.NewContext(req, res)
-	WWWRedirect()(next)(c)
+	res := redirectTest(WWWRedirect, "labstack.com", nil)
+
 	assert.Equal(t, http.StatusMovedPermanently, res.Code)
 	assert.Equal(t, "http://www.labstack.com/", res.Header().Get(echo.HeaderLocation))
 }
 
 func TestRedirectNonWWWRedirect(t *testing.T) {
+	res := redirectTest(NonWWWRedirect, "www.labstack.com", nil)
+
+	assert.Equal(t, http.StatusMovedPermanently, res.Code)
+	assert.Equal(t, "http://labstack.com/", res.Header().Get(echo.HeaderLocation))
+}
+
+func redirectTest(fn middlewareGenerator, host string, header http.Header) *httptest.ResponseRecorder {
 	e := echo.New()
 	next := func(c echo.Context) (err error) {
 		return c.NoContent(http.StatusOK)
 	}
 	req := httptest.NewRequest(echo.GET, "/", nil)
-	req.Host = "www.labstack.com"
+	req.Host = host
+	if header != nil {
+		req.Header = header
+	}
 	res := httptest.NewRecorder()
 	c := e.NewContext(req, res)
-	NonWWWRedirect()(next)(c)
-	assert.Equal(t, http.StatusMovedPermanently, res.Code)
-	assert.Equal(t, "http://labstack.com/", res.Header().Get(echo.HeaderLocation))
+
+	fn()(next)(c)
+
+	return res
 }
