@@ -12,7 +12,7 @@ type (
 
 		// Status code to be used when redirecting the request.
 		// Optional, but when provided the request is redirected using this code.
-		RedirectCode int `json:"redirect_code"`
+		RedirectCode int `yaml:"redirect_code"`
 	}
 )
 
