@@ -3,13 +3,174 @@ package echo
 import (
 	"fmt"
 	"net/http"
+	"strings"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
 )
 
 var (
-	api = []Route{
+	staticRoutes = []*Route{
+		{"GET", "/", ""},
+		{"GET", "/cmd.html", ""},
+		{"GET", "/code.html", ""},
+		{"GET", "/contrib.html", ""},
+		{"GET", "/contribute.html", ""},
+		{"GET", "/debugging_with_gdb.html", ""},
+		{"GET", "/docs.html", ""},
+		{"GET", "/effective_go.html", ""},
+		{"GET", "/files.log", ""},
+		{"GET", "/gccgo_contribute.html", ""},
+		{"GET", "/gccgo_install.html", ""},
+		{"GET", "/go-logo-black.png", ""},
+		{"GET", "/go-logo-blue.png", ""},
+		{"GET", "/go-logo-white.png", ""},
+		{"GET", "/go1.1.html", ""},
+		{"GET", "/go1.2.html", ""},
+		{"GET", "/go1.html", ""},
+		{"GET", "/go1compat.html", ""},
+		{"GET", "/go_faq.html", ""},
+		{"GET", "/go_mem.html", ""},
+		{"GET", "/go_spec.html", ""},
+		{"GET", "/help.html", ""},
+		{"GET", "/ie.css", ""},
+		{"GET", "/install-source.html", ""},
+		{"GET", "/install.html", ""},
+		{"GET", "/logo-153x55.png", ""},
+		{"GET", "/Makefile", ""},
+		{"GET", "/root.html", ""},
+		{"GET", "/share.png", ""},
+		{"GET", "/sieve.gif", ""},
+		{"GET", "/tos.html", ""},
+		{"GET", "/articles/", ""},
+		{"GET", "/articles/go_command.html", ""},
+		{"GET", "/articles/index.html", ""},
+		{"GET", "/articles/wiki/", ""},
+		{"GET", "/articles/wiki/edit.html", ""},
+		{"GET", "/articles/wiki/final-noclosure.go", ""},
+		{"GET", "/articles/wiki/final-noerror.go", ""},
+		{"GET", "/articles/wiki/final-parsetemplate.go", ""},
+		{"GET", "/articles/wiki/final-template.go", ""},
+		{"GET", "/articles/wiki/final.go", ""},
+		{"GET", "/articles/wiki/get.go", ""},
+		{"GET", "/articles/wiki/http-sample.go", ""},
+		{"GET", "/articles/wiki/index.html", ""},
+		{"GET", "/articles/wiki/Makefile", ""},
+		{"GET", "/articles/wiki/notemplate.go", ""},
+		{"GET", "/articles/wiki/part1-noerror.go", ""},
+		{"GET", "/articles/wiki/part1.go", ""},
+		{"GET", "/articles/wiki/part2.go", ""},
+		{"GET", "/articles/wiki/part3-errorhandling.go", ""},
+		{"GET", "/articles/wiki/part3.go", ""},
+		{"GET", "/articles/wiki/test.bash", ""},
+		{"GET", "/articles/wiki/test_edit.good", ""},
+		{"GET", "/articles/wiki/test_Test.txt.good", ""},
+		{"GET", "/articles/wiki/test_view.good", ""},
+		{"GET", "/articles/wiki/view.html", ""},
+		{"GET", "/codewalk/", ""},
+		{"GET", "/codewalk/codewalk.css", ""},
+		{"GET", "/codewalk/codewalk.js", ""},
+		{"GET", "/codewalk/codewalk.xml", ""},
+		{"GET", "/codewalk/functions.xml", ""},
+		{"GET", "/codewalk/markov.go", ""},
+		{"GET", "/codewalk/markov.xml", ""},
+		{"GET", "/codewalk/pig.go", ""},
+		{"GET", "/codewalk/popout.png", ""},
+		{"GET", "/codewalk/run", ""},
+		{"GET", "/codewalk/sharemem.xml", ""},
+		{"GET", "/codewalk/urlpoll.go", ""},
+		{"GET", "/devel/", ""},
+		{"GET", "/devel/release.html", ""},
+		{"GET", "/devel/weekly.html", ""},
+		{"GET", "/gopher/", ""},
+		{"GET", "/gopher/appenginegopher.jpg", ""},
+		{"GET", "/gopher/appenginegophercolor.jpg", ""},
+		{"GET", "/gopher/appenginelogo.gif", ""},
+		{"GET", "/gopher/bumper.png", ""},
+		{"GET", "/gopher/bumper192x108.png", ""},
+		{"GET", "/gopher/bumper320x180.png", ""},
+		{"GET", "/gopher/bumper480x270.png", ""},
+		{"GET", "/gopher/bumper640x360.png", ""},
+		{"GET", "/gopher/doc.png", ""},
+		{"GET", "/gopher/frontpage.png", ""},
+		{"GET", "/gopher/gopherbw.png", ""},
+		{"GET", "/gopher/gophercolor.png", ""},
+		{"GET", "/gopher/gophercolor16x16.png", ""},
+		{"GET", "/gopher/help.png", ""},
+		{"GET", "/gopher/pkg.png", ""},
+		{"GET", "/gopher/project.png", ""},
+		{"GET", "/gopher/ref.png", ""},
+		{"GET", "/gopher/run.png", ""},
+		{"GET", "/gopher/talks.png", ""},
+		{"GET", "/gopher/pencil/", ""},
+		{"GET", "/gopher/pencil/gopherhat.jpg", ""},
+		{"GET", "/gopher/pencil/gopherhelmet.jpg", ""},
+		{"GET", "/gopher/pencil/gophermega.jpg", ""},
+		{"GET", "/gopher/pencil/gopherrunning.jpg", ""},
+		{"GET", "/gopher/pencil/gopherswim.jpg", ""},
+		{"GET", "/gopher/pencil/gopherswrench.jpg", ""},
+		{"GET", "/play/", ""},
+		{"GET", "/play/fib.go", ""},
+		{"GET", "/play/hello.go", ""},
+		{"GET", "/play/life.go", ""},
+		{"GET", "/play/peano.go", ""},
+		{"GET", "/play/pi.go", ""},
+		{"GET", "/play/sieve.go", ""},
+		{"GET", "/play/solitaire.go", ""},
+		{"GET", "/play/tree.go", ""},
+		{"GET", "/progs/", ""},
+		{"GET", "/progs/cgo1.go", ""},
+		{"GET", "/progs/cgo2.go", ""},
+		{"GET", "/progs/cgo3.go", ""},
+		{"GET", "/progs/cgo4.go", ""},
+		{"GET", "/progs/defer.go", ""},
+		{"GET", "/progs/defer.out", ""},
+		{"GET", "/progs/defer2.go", ""},
+		{"GET", "/progs/defer2.out", ""},
+		{"GET", "/progs/eff_bytesize.go", ""},
+		{"GET", "/progs/eff_bytesize.out", ""},
+		{"GET", "/progs/eff_qr.go", ""},
+		{"GET", "/progs/eff_sequence.go", ""},
+		{"GET", "/progs/eff_sequence.out", ""},
+		{"GET", "/progs/eff_unused1.go", ""},
+		{"GET", "/progs/eff_unused2.go", ""},
+		{"GET", "/progs/error.go", ""},
+		{"GET", "/progs/error2.go", ""},
+		{"GET", "/progs/error3.go", ""},
+		{"GET", "/progs/error4.go", ""},
+		{"GET", "/progs/go1.go", ""},
+		{"GET", "/progs/gobs1.go", ""},
+		{"GET", "/progs/gobs2.go", ""},
+		{"GET", "/progs/image_draw.go", ""},
+		{"GET", "/progs/image_package1.go", ""},
+		{"GET", "/progs/image_package1.out", ""},
+		{"GET", "/progs/image_package2.go", ""},
+		{"GET", "/progs/image_package2.out", ""},
+		{"GET", "/progs/image_package3.go", ""},
+		{"GET", "/progs/image_package3.out", ""},
+		{"GET", "/progs/image_package4.go", ""},
+		{"GET", "/progs/image_package4.out", ""},
+		{"GET", "/progs/image_package5.go", ""},
+		{"GET", "/progs/image_package5.out", ""},
+		{"GET", "/progs/image_package6.go", ""},
+		{"GET", "/progs/image_package6.out", ""},
+		{"GET", "/progs/interface.go", ""},
+		{"GET", "/progs/interface2.go", ""},
+		{"GET", "/progs/interface2.out", ""},
+		{"GET", "/progs/json1.go", ""},
+		{"GET", "/progs/json2.go", ""},
+		{"GET", "/progs/json2.out", ""},
+		{"GET", "/progs/json3.go", ""},
+		{"GET", "/progs/json4.go", ""},
+		{"GET", "/progs/json5.go", ""},
+		{"GET", "/progs/run", ""},
+		{"GET", "/progs/slices.go", ""},
+		{"GET", "/progs/timeout1.go", ""},
+		{"GET", "/progs/timeout2.go", ""},
+		{"GET", "/progs/update.bash", ""},
+	}
+
+	gitHubAPI = []*Route{
 		// OAuth Authorizations
 		{"GET", "/authorizations", ""},
 		{"GET", "/authorizations/:id", ""},
@@ -271,6 +432,73 @@ var (
 		//{"PATCH", "/user/keys/:id", ""},
 		{"DELETE", "/user/keys/:id", ""},
 	}
+
+	parseAPI = []*Route{
+		// Objects
+		{"POST", "/1/classes/:className", ""},
+		{"GET", "/1/classes/:className/:objectId", ""},
+		{"PUT", "/1/classes/:className/:objectId", ""},
+		{"GET", "/1/classes/:className", ""},
+		{"DELETE", "/1/classes/:className/:objectId", ""},
+
+		// Users
+		{"POST", "/1/users", ""},
+		{"GET", "/1/login", ""},
+		{"GET", "/1/users/:objectId", ""},
+		{"PUT", "/1/users/:objectId", ""},
+		{"GET", "/1/users", ""},
+		{"DELETE", "/1/users/:objectId", ""},
+		{"POST", "/1/requestPasswordReset", ""},
+
+		// Roles
+		{"POST", "/1/roles", ""},
+		{"GET", "/1/roles/:objectId", ""},
+		{"PUT", "/1/roles/:objectId", ""},
+		{"GET", "/1/roles", ""},
+		{"DELETE", "/1/roles/:objectId", ""},
+
+		// Files
+		{"POST", "/1/files/:fileName", ""},
+
+		// Analytics
+		{"POST", "/1/events/:eventName", ""},
+
+		// Push Notifications
+		{"POST", "/1/push", ""},
+
+		// Installations
+		{"POST", "/1/installations", ""},
+		{"GET", "/1/installations/:objectId", ""},
+		{"PUT", "/1/installations/:objectId", ""},
+		{"GET", "/1/installations", ""},
+		{"DELETE", "/1/installations/:objectId", ""},
+
+		// Cloud Functions
+		{"POST", "/1/functions", ""},
+	}
+
+	googlePlusAPI = []*Route{
+		// People
+		{"GET", "/people/:userId", ""},
+		{"GET", "/people", ""},
+		{"GET", "/activities/:activityId/people/:collection", ""},
+		{"GET", "/people/:userId/people/:collection", ""},
+		{"GET", "/people/:userId/openIdConnect", ""},
+
+		// Activities
+		{"GET", "/people/:userId/activities/:collection", ""},
+		{"GET", "/activities/:activityId", ""},
+		{"GET", "/activities", ""},
+
+		// Comments
+		{"GET", "/activities/:activityId/comments", ""},
+		{"GET", "/comments/:commentId", ""},
+
+		// Moments
+		{"POST", "/people/:userId/moments/:collection", ""},
+		{"GET", "/people/:userId/moments/:collection", ""},
+		{"DELETE", "/moments/:id", ""},
+	}
 )
 
 func TestRouterStatic(t *testing.T) {
@@ -295,7 +523,7 @@ func TestRouterParam(t *testing.T) {
 	})
 	c := e.NewContext(nil, nil).(*context)
 	r.Find(GET, "/users/1", c)
-	assert.Equal(t, "1", c.P(0))
+	assert.Equal(t, "1", c.Param("id"))
 }
 
 func TestRouterTwoParam(t *testing.T) {
@@ -307,8 +535,8 @@ func TestRouterTwoParam(t *testing.T) {
 	c := e.NewContext(nil, nil).(*context)
 
 	r.Find(GET, "/users/1/files/1", c)
-	assert.Equal(t, "1", c.P(0))
-	assert.Equal(t, "1", c.P(1))
+	assert.Equal(t, "1", c.Param("uid"))
+	assert.Equal(t, "1", c.Param("fid"))
 }
 
 // Issue #378
@@ -347,13 +575,40 @@ func TestRouterMatchAny(t *testing.T) {
 	c := e.NewContext(nil, nil).(*context)
 
 	r.Find(GET, "/", c)
-	assert.Equal(t, "", c.P(0))
+	assert.Equal(t, "", c.Param("*"))
 
 	r.Find(GET, "/download", c)
-	assert.Equal(t, "download", c.P(0))
+	assert.Equal(t, "download", c.Param("*"))
 
 	r.Find(GET, "/users/joe", c)
-	assert.Equal(t, "joe", c.P(0))
+	assert.Equal(t, "joe", c.Param("*"))
+}
+
+func TestRouterMatchAnyMultiLevel(t *testing.T) {
+	e := New()
+	r := e.router
+	handler := func(c Context) error {
+		c.Set("path", c.Path())
+		return nil
+	}
+
+	// Routes
+	r.Add(GET, "/api/users/jack", handler)
+	r.Add(GET, "/api/users/jill", handler)
+	r.Add(GET, "/*", handler)
+
+	c := e.NewContext(nil, nil).(*context)
+	r.Find(GET, "/api/users/jack", c)
+	c.handler(c)
+	assert.Equal(t, "/api/users/jack", c.Get("path"))
+
+	r.Find(GET, "/api/users/jill", c)
+	c.handler(c)
+	assert.Equal(t, "/api/users/jill", c.Get("path"))
+
+	r.Find(GET, "/api/users/joe", c)
+	c.handler(c)
+	assert.Equal(t, "/*", c.Get("path"))
 }
 
 func TestRouterMicroParam(t *testing.T) {
@@ -364,9 +619,9 @@ func TestRouterMicroParam(t *testing.T) {
 	})
 	c := e.NewContext(nil, nil).(*context)
 	r.Find(GET, "/1/2/3", c)
-	assert.Equal(t, "1", c.P(0))
-	assert.Equal(t, "2", c.P(1))
-	assert.Equal(t, "3", c.P(2))
+	assert.Equal(t, "1", c.Param("a"))
+	assert.Equal(t, "2", c.Param("b"))
+	assert.Equal(t, "3", c.Param("c"))
 }
 
 func TestRouterMixParamMatchAny(t *testing.T) {
@@ -381,7 +636,7 @@ func TestRouterMixParamMatchAny(t *testing.T) {
 
 	r.Find(GET, "/users/joe/comments", c)
 	c.handler(c)
-	assert.Equal(t, "joe", c.P(0))
+	assert.Equal(t, "joe", c.Param("id"))
 }
 
 func TestRouterMultiRoute(t *testing.T) {
@@ -405,7 +660,7 @@ func TestRouterMultiRoute(t *testing.T) {
 
 	// Route > /users/:id
 	r.Find(GET, "/users/1", c)
-	assert.Equal(t, "1", c.P(0))
+	assert.Equal(t, "1", c.Param("id"))
 
 	// Route > /user
 	c = e.NewContext(nil, nil).(*context)
@@ -483,7 +738,7 @@ func TestRouterPriority(t *testing.T) {
 	r.Find(GET, "/users/joe/books", c)
 	c.handler(c)
 	assert.Equal(t, 7, c.Get("g"))
-	assert.Equal(t, "joe/books", c.Param("_*"))
+	assert.Equal(t, "joe/books", c.Param("*"))
 }
 
 // Issue #372
@@ -542,14 +797,14 @@ func TestRouterParamNames(t *testing.T) {
 	// Route > /users/:id
 	r.Find(GET, "/users/1", c)
 	assert.Equal(t, "id", c.pnames[0])
-	assert.Equal(t, "1", c.P(0))
+	assert.Equal(t, "1", c.Param("id"))
 
 	// Route > /users/:uid/files/:fid
 	r.Find(GET, "/users/1/files/1", c)
 	assert.Equal(t, "uid", c.pnames[0])
-	assert.Equal(t, "1", c.P(0))
+	assert.Equal(t, "1", c.Param("uid"))
 	assert.Equal(t, "fid", c.pnames[1])
-	assert.Equal(t, "1", c.P(1))
+	assert.Equal(t, "1", c.Param("fid"))
 }
 
 // Issue #623
@@ -584,7 +839,7 @@ func TestRouterStaticDynamicConflict(t *testing.T) {
 	assert.Equal(t, 3, c.Get("c"))
 }
 
-func TestRouterAPI(t *testing.T) {
+func testRouterAPI(t *testing.T, api []*Route) {
 	e := New()
 	r := e.router
 
@@ -596,21 +851,36 @@ func TestRouterAPI(t *testing.T) {
 	c := e.NewContext(nil, nil).(*context)
 	for _, route := range api {
 		r.Find(route.Method, route.Path, c)
-		for i, n := range c.pnames {
-			if assert.NotEmpty(t, n) {
-				assert.Equal(t, ":"+n, c.P(i))
+		tokens := strings.Split(route.Path[1:], "/")
+		for _, token := range tokens {
+			if token[0] == ':' {
+				assert.Equal(t, c.Param(token[1:]), token)
 			}
 		}
 	}
 }
 
-func BenchmarkRouterGitHubAPI(b *testing.B) {
+func TestRouterGitHubAPI(t *testing.T) {
+	testRouterAPI(t, gitHubAPI)
+}
+
+// Issue #729
+func TestRouterParamAlias(t *testing.T) {
+	api := []*Route{
+		{GET, "/users/:userID/following", ""},
+		{GET, "/users/:userID/followedBy", ""},
+		{GET, "/users/:userID/follow", ""},
+	}
+	testRouterAPI(t, api)
+}
+
+func benchmarkRouterRoutes(b *testing.B, routes []*Route) {
 	e := New()
 	r := e.router
 	b.ReportAllocs()
 
 	// Add routes
-	for _, route := range api {
+	for _, route := range routes {
 		r.Add(route.Method, route.Path, func(c Context) error {
 			return nil
 		})
@@ -618,17 +888,30 @@ func BenchmarkRouterGitHubAPI(b *testing.B) {
 
 	// Find routes
 	for i := 0; i < b.N; i++ {
-		for _, route := range api {
-			// c := e.pool.Get().(*context)
-			c := e.AcquireContext()
+		for _, route := range gitHubAPI {
+			c := e.pool.Get().(*context)
 			r.Find(route.Method, route.Path, c)
-			// router.Find(r.Method, r.Path, c)
-			e.ReleaseContext(c)
-			// e.pool.Put(c)
+			e.pool.Put(c)
 		}
 	}
 }
 
+func BenchmarkRouterStaticRoutes(b *testing.B) {
+	benchmarkRouterRoutes(b, staticRoutes)
+}
+
+func BenchmarkRouterGitHubAPI(b *testing.B) {
+	benchmarkRouterRoutes(b, gitHubAPI)
+}
+
+func BenchmarkRouterParseAPI(b *testing.B) {
+	benchmarkRouterRoutes(b, parseAPI)
+}
+
+func BenchmarkRouterGooglePlusAPI(b *testing.B) {
+	benchmarkRouterRoutes(b, googlePlusAPI)
+}
+
 func (n *node) printTree(pfx string, tail bool) {
 	p := prefix(tail, pfx, "└── ", "├── ")
 	fmt.Printf("%s%s, %p: type=%d, parent=%p, handler=%v\n", p, n.prefix, n, n.kind, n.parent, n.methodHandler)
